@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -65,11 +64,14 @@ file_sel_response_cb (GtkWidget      *widget,
 	current_folder = gtk_file_chooser_get_current_folder_uri (file_sel);
 	uri = gtk_file_chooser_get_uri (file_sel);
 
-	g_settings_set_string (data->settings, PREF_ADD_CURRENT_FOLDER, current_folder);
-	g_settings_set_string (data->settings, PREF_ADD_FILENAME, uri);
-	fr_window_set_add_default_dir (window, current_folder);
-
-	g_free (uri);
+	if (current_folder != NULL) {
+		g_settings_set_string (data->settings, PREF_ADD_CURRENT_FOLDER, current_folder);
+		fr_window_set_add_default_dir (window, current_folder);
+	}
+	if (uri != NULL) {
+		g_settings_set_string (data->settings, PREF_ADD_FILENAME, uri);
+		g_free (uri);
+	}
 
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
 		gtk_widget_destroy (data->dialog);
