@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -39,7 +38,11 @@
 #include "fr-error.h"
 #include "fr-marshal.h"
 #include "fr-process.h"
-#include "main.h"
+#include "fr-init.h"
+
+#if ENABLE_MAGIC
+#include <magic.h>
+#endif
 
 #ifndef NCARGS
 #define NCARGS _POSIX_ARG_MAX
@@ -525,43 +528,72 @@ get_mime_type_from_content (GFile *file)
 static const char *
 get_mime_type_from_magic_numbers (GFile *file)
 {
-	static struct {
-		const char *mime_type;
-		const char *first_bytes;
-		int         offset;
-		int         len;
-	} sniffer_data [] = {
-		/* Magic numbers taken from magic/Magdir/archive from the
-		 * file-4.21 tarball. */
-		{ "application/x-7z-compressed", "7z\274\257\047\034", 0, 5 },
-		{ "application/x-ace", "**ACE**", 7, 7 },
-		{ "application/x-arj", "\x60\xea", 0, 2 },
-		{ "application/x-bzip2", "BZh", 0, 3 },
-		{ "application/x-gzip", "\037\213", 0, 2 },
-		{ "application/x-lzip", "LZIP", 0, 4 },
-		{ "application/x-lzop", "\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a", 0, 9 },
-		{ "application/x-rar", "Rar!", 0, 4 },
-		{ "application/x-rzip", "RZIP", 0, 4 },
-		{ "application/x-xz", "\3757zXZ\000", 0, 6 },
-		{ "application/x-zoo", "\xdc\xa7\xc4\xfd", 20, 4 },
-		{ "application/zip", "PK\003\004", 0, 4 },
-		{ "application/zip", "PK00PK\003\004", 0, 8 },
-		{ "application/x-lrzip", "LRZI", 0, 4 },
-		{ NULL, NULL, 0 }
+#if ENABLE_MAGIC
+	static magic_t magic = NULL;
+
+	if (! magic) {
+		magic = magic_open (MAGIC_MIME);
+		if (magic)
+			magic_load (magic, NULL);
+		else
+			g_warning ("unable to open magic database");
+	}
+
+	if (magic) {
+		const char * mime_type = NULL;
+		char * filepath = g_file_get_path (file);
+
+		if (filepath) {
+			mime_type = magic_file (magic, filepath);
+			g_free (filepath);
+		}
+
+		if (mime_type)
+			return mime_type;
+
+		g_warning ("unable to detect filetype from magic: %s",
+			   magic_error (magic));
+	}
+#else
+	static const struct magic {
+		const unsigned int off;
+		const unsigned int len;
+		const char * const id;
+		const char * const mime_type;
+	} magic_ids [] = {
+		/* magic ids taken from magic/Magdir/archive from the file-4.21 tarball */
+		{ 0,  6, "7z\274\257\047\034",                   "application/x-7z-compressed" },
+		{ 7,  7, "**ACE**",                              "application/x-ace"           },
+		{ 0,  2, "\x60\xea",                             "application/x-arj"           },
+		{ 0,  3, "BZh",                                  "application/x-bzip2"         },
+		{ 0,  2, "\037\213",                             "application/x-gzip"          },
+		{ 0,  4, "LZIP",                                 "application/x-lzip"          },
+		{ 0,  9, "\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a", "application/x-lzop",         },
+		{ 0,  4, "Rar!",                                 "application/x-rar"           },
+		{ 0,  4, "RZIP",                                 "application/x-rzip"          },
+		{ 0,  6, "\3757zXZ\000",                         "application/x-xz"            },
+		{ 20, 4, "\xdc\xa7\xc4\xfd",                     "application/x-zoo",          },
+		{ 0,  4, "PK\003\004",                           "application/zip"             },
+		{ 0,  8, "PK00PK\003\004",                       "application/zip"             },
+		{ 0,  4, "LRZI",                                 "application/x-lrzip"         },
 	};
+
 	char buffer[32];
 	int  i;
 
-	if (! g_load_file_in_buffer (file, buffer, 32, NULL))
+	if (! g_load_file_in_buffer (file, buffer, sizeof (buffer), NULL))
 		return NULL;
 
-	for (i = 0; sniffer_data[i].mime_type != NULL; i++)
-		if (memcmp (sniffer_data[i].first_bytes,
-			    buffer + sniffer_data[i].offset,
-			    sniffer_data[i].len) == 0)
-		{
-			return sniffer_data[i].mime_type;
-		}
+	for (i = 0; i < G_N_ELEMENTS (magic_ids); i++) {
+		const struct magic * const magic = &magic_ids[i];
+
+		if (sizeof (buffer) < (magic->off + magic->len))
+			g_warning ("buffer underrun for mime type '%s' magic",
+				   magic->mime_type);
+		else if (! memcmp (buffer + magic->off, magic->id, magic->len))
+			return magic->mime_type;
+	}
+#endif
 
 	return NULL;
 }
@@ -1448,25 +1480,6 @@ fr_archive_set_add_is_stoppable_func (FrArchive     *archive,
 }
 
 
-static GList *
-convert_to_local_file_list (GList *file_list)
-{
-	GList *local_file_list = NULL;
-	GList *scan;
-
-	for (scan = file_list; scan; scan = scan->next) {
-		char *uri = scan->data;
-		char *local_filename;
-
-		local_filename = g_uri_unescape_string (uri, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH);
-		if (local_filename != NULL)
-			local_file_list = g_list_prepend (local_file_list, local_filename);
-	}
-
-	return local_file_list;
-}
-
-
 static gboolean
 save_list_to_temp_file (GList   *file_list,
 		        char   **list_dir,
@@ -1591,7 +1604,6 @@ fr_archive_add (FrArchive     *archive,
 
 	fr_archive_stoppable (archive, fr_archive_add_is_stoppable (archive));
 
-	file_list = convert_to_local_file_list (file_list);
 	tmp_base_dir = g_strdup (base_dir);
 
 	if ((dest_dir != NULL) && (*dest_dir != '\0') && (strcmp (dest_dir, "/") != 0)) {
@@ -1608,10 +1620,9 @@ fr_archive_add (FrArchive     *archive,
 			char *filename = scan->data;
 			new_file_list = g_list_prepend (new_file_list, g_build_filename (rel_dest_dir, filename, NULL));
 		}
-		path_list_free (file_list);
 	}
 	else
-		new_file_list = file_list;
+		new_file_list = path_list_dup (file_list);
 
 	/* if the command cannot update,  get the list of files that are
 	 * newer than the ones in the archive. */
